import os

from ROTools.Helpers.Attr import setattr_ex, getattr_ex
from ROTools.Helpers.DictObj import DictObj

class Config(DictObj):
    def __init__(self, **kwargs):
        super().__init__()
        for k in kwargs.keys():
            setattr(self, k, kwargs[k])

    def add_env_data(self, prefix):
        extra_config = [(key[len(prefix):].lower(), value) for key, value in os.environ.items() if key.startswith(prefix)]

        for key, value in extra_config:
            _old_value = getattr_ex(self, key)
            if _old_value is not None and isinstance(_old_value, bool):
                setattr_ex(self, key, value in ["true", "True", "TRUE", "1", ])
                continue

            if _old_value is not None and isinstance(_old_value, int):
                setattr_ex(self, key, int(value))
                continue

            if _old_value is not None and isinstance(_old_value, float):
                setattr_ex(self, key, float(value))
                continue

            if _old_value is not None:
                setattr_ex(self, key, value)

    def set_common_override(self, source_path, target_path):
        _common_override = self.get(source_path) or DictObj()
        values_to_override = _common_override.fields_list()

        for element in self.get(target_path).values():
            element.set_values(values_to_override, only_if_exists=False)
        self.rem(source_path)

    def set_common_default(self, source_path, target_path):
        _common_default = self.get(source_path) or DictObj()
        values_default = _common_default.fields_list()

        for element in self.get(target_path).values():
            for path, nev_value in values_default:
                if element.get(path) is None:
                    element.set(path, nev_value)
        self.rem(source_path)

    def dump_config(self):
        print()
        print("========")
        self.dump()
        print("========")
        print()
